# Summary

- [OpenMLDB 文档](README.md)
- [关于 OpenMLDB](about/about.md)
  - [OpenMLDB 简介](about/intro.md)
  - [发展历程](about/milestones.md)
  - [v0.5.x Release Notes](about/release_notes.md)
- [快速上手](quickstart/quickstart.md)
  - [OpenMLDB 快速上手](quickstart/openmldb_quickstart.md)
  - [Java SDK 快速上手](quickstart/java_sdk.md)
  - [Python SDK快速上手](./quickstart/python_sdk.md)
- [教程](tutorial/tutorial.md)
  - [OpenMLDB集群版和单机版区别](tutorial/standalone_vs_cluster.md)
  - [OpenMLDB数据导入工具使用](./tutorial/data_import.md)
  - [面向特征工程优化的OpenMLDB Spark发行版](tutorial/openmldbspark_distribution.md)
  - [深入浅出特征工程 -- 基于 OpenMLDB 的实践指南（上）](tutorial/tutorial_sql_1.md)
  - [深入浅出特征工程 -- 基于 OpenMLDB 的实践指南（下）](tutorial/tutorial_sql_2.md)
- [使用案例](use_case/use_case.md)
  - [案例1：出租车行车时间预测](use_case/taxi_tour_duration_prediction.md)
- [安装部署](deploy/deploy.md)
  - [编译OpenMLDB](deploy/compile.md)
  - [安装部署详细说明](deploy/install_deploy.md)
  - [配置文件说明](deploy/conf.md)
- [运维](maintain/maintain.md)
  - [升级](maintain/upgrade.md)
  - [备份](maintain/backup.md)
  - [扩缩容](maintain/scale.md)
  - [监控](maintain/monitoring.md)
- [开发者文档](developer/developer.md)
  - [Contributing](./developer/contributing.md)
  - [OpenMLDB SQL 内置函数开发指南](developer/built_in_function_develop_guide.md)
  - [OpenMLDB SQL 自定义函数开发指南](developer/udf_develop_guide.md)
- [参考](reference/reference.md)
  - [OpenMLDB运维CLI](reference/cli.md)
  - [REST APIs](reference/rest_api.md)
  - [SQL](reference/sql/reference.md)
    - [语言结构](reference/sql/language_structure/reference.md)
      - [关键字](reference/sql/language_structure/keywords.md)
      - [对象名和标识符](reference/sql/language_structure/object_names_identifiers.md)
      - [字面量](reference/sql/language_structure/literals_value.md)
      - [大小写敏感](reference/sql/language_structure/case_sensitive.md)
      - [注释](reference/sql/language_structure/comment.md)
    - [数据类型](reference/sql/data_types/reference.md)
      - [数值类型](reference/sql/data_types/numeric_types.md)
      - [字符串类型](reference/sql/data_types/string_types.md)
      - [日期和时间类型](reference/sql/data_types/date_and_time_types.md)
    - [表达式，函数和运算](reference/sql/functions_and_operators/reference.md)
      - [表达式运算](reference/sql/functions_and_operators/operators.md)
      - [内置函数](reference/sql/functions_and_operators/Files/udfs_8h.md)
    - [数据查询语句(DQL)](reference/sql/dql/reference.md)
      - [SELECT查询语句概况](reference/sql/dql/SELECT_STATEMENT.md)
      - [JOIN子句](reference/sql/dql/JOIN_CLAUSE.md)
      - [WHERE子句](reference/sql/dql/WHERE_CLAUSE.md)
      - [GROUP BY子句](reference/sql/dql/GROUP_BY_CLAUSE.md)
      - [HAVING 子句](reference/sql/dql/HAVING_CLAUSE.md)
      - [WINDOW 子句](reference/sql/dql/WINDOW_CLAUSE.md)
      - [LIMIT子句](reference/sql/dql/LIMIT_CLAUSE.md)
      - [无表SELECT查询语句](reference/sql/dql/NO_TABLE_SELECT_CLAUSE.md)
      - [SELECT INTO语句](reference/sql/dql/SELECT_INTO_STATEMENT.md)
    - [数据操作语句(DML)](reference/sql/dml/reference.md)
      - [插入语句](reference/sql/dml/INSERT_STATEMENT.md)
      - [LOAD DATA语句](reference/sql/dml/LOAD_DATA_STATEMENT.md)
    - [数据定义语句(DDL)](reference/sql/ddl/reference.md)
      - [创建数据库(CREATE DATABASE)](reference/sql/ddl/CREATE_DATABASE_STATEMENT.md)
      - [查看数据库列表(SHOW DATABASES)](reference/sql/ddl/SHOW_DATABASES_STATEMENT.md)
      - [USE数据库](reference/sql/ddl/USE_DATABASE_STATEMENT.md)
      - [删除数据库(DROP DATABASE)](reference/sql/ddl/DROP_DATABASE_STATEMENT.md)
      - [查看表详情(DESC)](reference/sql/ddl/DESC_STATEMENT.md)
      - [创建表(CREATE TABLE)](reference/sql/ddl/CREATE_TABLE_STATEMENT.md)
      - [删除表(DROP TABLE)](reference/sql/ddl/DROP_TABLE_STATEMENT.md)
      - [查看表列表(SHOW TABLES)](reference/sql/ddl/SHOW_TABLES_STATEMENT.md)
      - [查看系统变量(SHOW VARIABLES)](reference/sql/ddl/SHOW_VARIABLES_STATEMENT.md)
      - [SET语句(SET )](reference/sql/ddl/SET_STATEMENT.md)
    - [DEPLOYMENT 管理](reference/sql/deployment_manage/reference.md)
      - [创建DEPLOYMENT](reference/sql/deployment_manage/DEPLOY_STATEMENT.md)
      - [删除DEPLOYMENT(DROP DEPLOYMENT)](reference/sql/deployment_manage/DROP_DEPLOYMENT_STATEMENT.md)
      - [查看DEPLOYMENTS列表(SHOW DEPLOYMENTS)](reference/sql/deployment_manage/SHOW_DEPLOYMENTS.md)
      - [查看DEPLOYMENT详情(SHOW DEPLOYMENT)](reference/sql/deployment_manage/SHOW_DEPLOYMENT.md)
      - [OpenMLDB SQL上线规范和要求](reference/sql/deployment_manage/ONLINE_SERVING_REQUIREMENTS.md)
    - [任务管理](reference/sql/task_manage/reference.md)
      - [查看Job列表(SHOW JOBS)](reference/sql/task_manage/SHOW_JOBS.md)
      - [查看Job详情(SHOW JOB)](reference/sql/task_manage/SHOW_JOB.md)
      - [停止Job任务(STOP JOB)](reference/sql/task_manage/STOP_JOB.md)
      - [提交Job任务](reference/sql/task_manage/SUBMIT_JOB.md)
